/***************************************************************************
 *   Copyright (C) 2004 by Joseph Wenninger                                *
 *   jowenn@kde.org                                                        *
 *   (C) 2005 Fabian Franz                                                 *
 *            George Wright                                                *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <iostream>
#include <stdlib.h>

#include <qdir.h>
#include <kprocess.h>
#include <kdebug.h>
#include "knxconnection.h"
#include "winuserpwd.h"
#include <assert.h>
#include <qfile.h>
#include <sys/stat.h>
#include <sys/types.h>
#include "kcookie.h"
#include <klocale.h>
#include <qtimer.h>
#include <qdesktopwidget.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <kstandarddirs.h>
#include <kpassdlg.h>
#include <kmessagebox.h> //move the kmessagebox calles to the connecting dialog
#include <kinputdialog.h> // perhaps move to connecting dialog
#include <qstringlist.h>
#include "nxc/Logger.h"
#include "knxconnectionstatus.h"
#include "nxc/CallbackMessageTypes.h"

using namespace std;

KNXConnection::KNXConnection():QObject() {
}

KNXConnection::~KNXConnection() {
}

bool KNXConnection::doConnect(const QString &sessionName) {
	// Get some status info
	CallBack *callback = new CallBack;
	thread.NXDriverInstance.SetCallback(callback);

	connecting = new ConnectingDialog();
	connecting->show();

	connect (callback->status, SIGNAL(message(unsigned int, QString)), this, SLOT(output(unsigned int, QString)));
	thread.start();
 	return true;
}

void KNXConnection::output(unsigned int type, QString msg) {
	switch (type) {
		case CB_MSG_STATUS:
			kdDebug() << "Information: " << msg << endl;
			connecting->updateStatus("<b>" + msg + "</b>");
		break;
		case CB_MSG_SESSION:
			kdDebug() << "Session: " << msg << endl;
			connecting->updateStatus("<b>Connected to session.</b>");
			// connecting->close();
		break;
		case CB_MSG_COMPLETE: // This doesn't appear to be used (yet?)
			kdDebug() << "Completed." << endl;
		break;
		case CB_MSG_ERROR:
			kdDebug() << msg << endl;
			connecting->showError(msg);
		break;
	}
}
void KNXConnection::setNXLogin(const QString& password, const QString& username){
	if (!username.isEmpty()) 
		thread.NXDriverInstance.SetUserName(username.latin1());
	
	if (!password.isEmpty()) 
		thread.NXDriverInstance.SetClearPassword(password.latin1());
}


void KNXConnection::load(const QString &connectionFile) {
	thread.NXDriverInstance.SetConfigFilePath(connectionFile.latin1());
}

void KNXConnection::abortConnecting() {
}

void NXConnectionThread::run() {
	NXDriverInstance.Run();
}

#include "knxconnection.moc"

