/***************************************************************************
 *   Copyright (C) 2004 by Joseph Wenninger                                *
 *   jowenn@kde.org                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "knxconnectionsettingsdialog.h"
#include "connectionsettings_base.h"
#include <klocale.h>
#include <kcombobox.h>
#include <klineedit.h>
#include <kstandarddirs.h>
#include <qwidgetstack.h>
#include <kdebug.h>
#include <knuminput.h>
#include <qfile.h>
#include <qcheckbox.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

KNXConnectionSettingsDialog::KNXConnectionSettingsDialog(const QString& path, const QString& config): 
	KDialogBase(0,0,false,i18n("Connection Settings"),KDialogBase::Ok | KDialogBase::Cancel),m_path(path) {
	
	m_widget=new ConnectionSettings_base(this);
	setMainWidget(m_widget);
	if (!config.isEmpty()) {
		m_widget->connectionname->setText(config);
		load(path+"/"+config+".nxs");
	} else {
		m_widget->connectionname->setText(i18n("NewConnection"));
	}
	connect(m_widget->sessiontype,SIGNAL(activated(int)),this,SLOT(sessionTypeChanged(int)));
	connect(m_widget->vncrememberpassword,SIGNAL(toggled(bool)),this,SLOT(vncRememberPasswordChanged(bool)));
	connect(m_widget->winrememberpassword,SIGNAL(toggled(bool)),this,SLOT(winRememberPasswordChanged(bool)));
	connect(m_widget->winknxlogin,SIGNAL(toggled(bool)),this,SLOT(winKnxLoginChanged(bool)));
}

KNXConnectionSettingsDialog::~KNXConnectionSettingsDialog() {
}

void KNXConnectionSettingsDialog::sessionTypeChanged(int type) {
	m_widget->sessiontype_settings->raiseWidget(type);
}



void KNXConnectionSettingsDialog::vncRememberPasswordChanged(bool checked) {
	m_widget->vncpassword->setEnabled(checked);
}

void KNXConnectionSettingsDialog::winRememberPasswordChanged(bool checked) {
	m_widget->winpassword->setEnabled(checked);
}

void KNXConnectionSettingsDialog::winKnxLoginChanged(bool checked) {
	m_widget->winpassword->setEnabled(checked && m_widget->winrememberpassword->isChecked());
	m_widget->winrememberpassword->setEnabled(checked);
	m_widget->winusername->setEnabled(checked);
}




void KNXConnectionSettingsDialog::load(QString filename) {
        QFile f(QFile::encodeName(filename));
        f.open(IO_ReadOnly);
        QDomDocument doc;
        bool ok=doc.setContent(&f);
        f.close();
        if (!ok) return; //error handling / security here
	QDomElement docRoot=doc.documentElement();
	for(QDomNode group=docRoot.firstChild(); !group.isNull(); group=group.nextSibling()) {
                QDomElement groupEl=group.toElement();
                if ( (groupEl.isNull()) || (groupEl.tagName()!="group")) continue;
                QString groupName=groupEl.attribute("name");
		if (groupName=="General") readGeneral(groupEl);
		if (groupName=="VNC Session") readVNC(groupEl);
		if (groupName=="Windows Session") readWindows(groupEl);
	}
	sessionTypeChanged(m_widget->sessiontype->currentItem());
	vncRememberPasswordChanged(m_widget->vncrememberpassword->isChecked());
	winRememberPasswordChanged(m_widget->winrememberpassword->isChecked());
	winKnxLoginChanged(m_widget->winknxlogin->isChecked());
}



void KNXConnectionSettingsDialog::readGeneral(QDomElement &groupEl) {
	enum {Unix=0,Windows=2,VNC=3} session=Unix;
	enum {KDE=0,Custom=1} desktop=KDE;

	for (QDomNode prop=groupEl.firstChild(); !prop.isNull(); prop=prop.nextSibling()) {
		QDomElement propEl=prop.toElement();
		if ( (propEl.isNull() || propEl.tagName()!="option")) continue;
		QString key=propEl.attribute("key");
		kdDebug()<<"readGeneral: key="<<key<<endl;
#warning NOT YET		if (key=="Backingstore") m_sessionString+=" --backingstore=\""+propEl.attribute("value")+"\"";
		if (key=="Link speed") {
			QString speed=propEl.attribute("value");
			uint item;
			if (speed=="modem") item=0;
			else if (speed=="isdn") item=1;
			else if (speed=="adsl") item=2;
			else if (speed=="wan") item=3;
			else if (speed=="lan") item=4;
			else item=0;
			m_widget->nxcompression->setCurrentItem(item);
		}
		else if (key=="Server host") m_widget->servername->setText(propEl.attribute("value"));
		else if (key=="Server port") m_widget->serverport->setValue(propEl.attribute("value").toULong());
		else if (key=="Resolution") { 
			QString resolution=propEl.attribute("value");
			if (resolution=="fullscreen") m_widget->screenresolution->setCurrentItem(2);
			else 
				m_widget->screenresolution->setCurrentText(resolution);

		}
		else if (key=="Session") {
			QString val=propEl.attribute("value");
			if (val=="Unix") session=Unix;
			else if (val=="Windows") session=Windows;
			else if (val=="VNC") session=VNC;
			else session=Unix;
		}
		else if (key=="Desktop") {
			QString val=propEl.attribute("value");
			if (val=="Custom") desktop=Custom;
			desktop=KDE;			
		} 		
#warning support for the rest needed

	}

	if (session==Unix) m_widget->sessiontype->setCurrentItem(desktop);
	else m_widget->sessiontype->setCurrentItem(session);
}

#if 0
void KNXConnectionSettingsDialog::readAdvanced(QDomElement &groupEl) {
	for (QDomNode prop=groupEl.firstChild(); !prop.isNull(); prop=prop.nextSibling()) {
		QDomElement propEl=prop.toElement();
		if ( (propEl.isNull() || propEl.tagName()!="option")) continue;
		QString key=propEl.attribute("key");
		if (key=="Cache size") m_sessionString+=" --cache=\""+propEl.attribute("value")+"M\"";
		else if (key=="Cache size on disk") m_sessionString+=" --images=\""+propEl.attribute("value")+"M\"";
		else if (key=="Enable SSL Encryption") m_sessionString+=" --encryption=\""+ QString((propEl.attribute("value")=="false")?"0":"1")+"\"";
#warning support for the rest needed
	}
}
#endif

static void KNXdecodePassword(QString pwd,QCString &result) {
	if (!pwd.isEmpty()) {
		QStringList cl=QStringList::split(':',pwd);
			int i=1;
			for (QStringList::Iterator ci=cl.begin();ci!=cl.end();i++,++ci) {
				XKeyEvent e;
				e.display=qt_xdisplay();
				e.keycode=XKeysymToKeycode(qt_xdisplay(),((*ci).toLong()-i));	
				char decoded[21];
				KeySym dummy;
				XComposeStatus dummy2;
				XLookupString(&e,decoded,20,&dummy,&dummy2);
				result+=decoded;
//				result+=XKeysymToString(((*ci).toLong()-i));
			}
		}

}

void KNXConnectionSettingsDialog::readVNC (QDomElement &groupEl) {
	QString displayNum="0";
	QString hostname="";
	QCString password;
	for (QDomNode prop=groupEl.firstChild(); !prop.isNull(); prop=prop.nextSibling()) {
		QDomElement propEl=prop.toElement();
		if ( (propEl.isNull() || propEl.tagName()!="option")) continue;
		QString key=propEl.attribute("key");
		if (key=="Display") displayNum=propEl.attribute("value");
		else if (key=="Server") hostname=propEl.attribute("value");
		else if (key=="Password") {
			KNXdecodePassword(propEl.attribute("value"),password);
			m_widget->vncpassword->setText(password);
		}
		else if (key=="Remember") m_widget->vncrememberpassword->setChecked(propEl.attribute("value")=="true");
	}

	m_widget->vncdisplay->setText(hostname+displayNum);
}

void KNXConnectionSettingsDialog::readWindows(QDomElement &groupEl)  {
	QCString password;
	for (QDomNode prop=groupEl.firstChild(); !prop.isNull(); prop=prop.nextSibling()) {
		QDomElement propEl=prop.toElement();
		if ( (propEl.isNull() || propEl.tagName()!="option")) continue;
		QString key=propEl.attribute("key");
		if (key=="Server") m_widget->winhost->setText(propEl.attribute("value"));
		else if (key=="Password") {
			KNXdecodePassword(propEl.attribute("value"),password);
			m_widget->winpassword->setText(password);
		}
		else if (key=="Remember") m_widget->winrememberpassword->setChecked(propEl.attribute("value")=="true");
		else if (key=="User") m_widget->winusername->setText(propEl.attribute("value"));
		else if (key="Authentication") m_widget->winknxlogin->setChecked(propEl.attribute("value")=="0");
	}
}

void KNXConnectionSettingsDialog::slotOk() {
	QDomDocument doc("NXClientSettings");
	QDomElement docRoot=doc.createElement("NXClientSettings");
	doc.appendChild(docRoot);
	docRoot.setAttribute("application","nxclient");
	docRoot.setAttribute("version","1.3");
	/*General group */
	QDomElement general=doc.createElement("group");
	general.setAttribute("name","General");
	docRoot.appendChild(general);
	writeGeneral(general);


	/*perhaps only the data for the current selected session type should be saved */

	/*windows*/
	QDomElement windows=doc.createElement("group");
	windows.setAttribute("name","Windows Session");
	docRoot.appendChild(windows);
	writeWindows(windows);
	/*vnc*/
	QDomElement vnc=doc.createElement("group");
	vnc.setAttribute("name","VNC Session");
	docRoot.appendChild(vnc);
	writeVNC(vnc);

	QString file(QFile::encodeName(m_path+m_widget->connectionname->text()+".nxs"));
	QFile f(file);
	KStandardDirs::makeDir(file.left(file.findRev("/")),0700);
	if (!f.open(IO_WriteOnly)) {
		KStandardDirs::makeDir(file.left(file.findRev("/")),0700);
		if (!f.open(IO_WriteOnly)) {
			//Error here
			return;
		}
	}
	QTextStream s(&f);
	s<<doc.toString();
	f.close();
	emit filesModified();
	KDialogBase::slotOk();	
}


static void appendProperty(QDomElement &group,QString name, QString value) {
	QDomElement propEl=group.ownerDocument().createElement("option");
	propEl.setAttribute("key",name);
	propEl.setAttribute("value",value);
	group.appendChild(propEl);
}


void KNXConnectionSettingsDialog::writeGeneral(QDomElement &group) {
	switch (m_widget->nxcompression->currentItem()) {
		case 0:	appendProperty(group,"Link speed","modem");
			break;
		case 1:	appendProperty(group,"Link speed","isdn");
			break;
		case 2:	appendProperty(group,"Link speed","adsl");
			break;
		case 3:	appendProperty(group,"Link speed","wan");
			break;
		case 4:	appendProperty(group,"Link speed","lan");
			break;
	}
	appendProperty(group,"Server host",m_widget->servername->text());
	appendProperty(group,"Server port",QString::number(m_widget->serverport->value()));
	if (m_widget->screenresolution->currentItem()==2)
		appendProperty(group,"Resolution","fullscreen");
	else appendProperty(group,"Resolution",m_widget->screenresolution->currentText());

	switch (m_widget->sessiontype->currentItem()) {
		case 0:	appendProperty(group,"Session","Unix");
			appendProperty(group,"Desktop","KDE");
			break;
		case 1:	appendProperty(group,"Session","Unix");
			appendProperty(group,"Desktop","Custom");
			break;
		case 2:	appendProperty(group,"Session","Windows");
			appendProperty(group,"Desktop","RDP");
			break;
		case 3:	appendProperty(group,"Session","VNC");
			appendProperty(group,"Desktop","RFB");
			break;

			
	}
}

void KNXConnectionSettingsDialog::writeVNC (QDomElement &group) {
	QString display=m_widget->vncdisplay->text();
	int colon=display.findRev(':');
	appendProperty(group,"Server",display.left(colon));
	appendProperty(group,"Display",display.right(display.length()-colon));
	appendProperty(group,"Remember",m_widget->vncrememberpassword->isChecked()?"true":"false");
	appendProperty(group,"Password","");
}

void KNXConnectionSettingsDialog::writeWindows(QDomElement &group)  {
	appendProperty(group,"Server",m_widget->winhost->text());
	appendProperty(group,"Authentication",m_widget->winknxlogin->isChecked()?"0":"1");
	appendProperty(group,"User",m_widget->winusername->text());
	appendProperty(group,"Remember",m_widget->winrememberpassword->isChecked()?"true":"false");
	appendProperty(group,"Password","");
}


#include "knxconnectionsettingsdialog.moc"
