/***************************************************************************
 *   Copyright (C) 2004 by Joseph Wenninger                                *
 *   jowenn@kde.org                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "logindialog.h"
#include "knxconnection.h"
#include "knxconnectionsettingsdialog.h"
#include <klocale.h>
#include <kcombobox.h>
#include <klineedit.h>
#include <kpushbutton.h>
#include <kapplication.h>
#include <qdir.h>
#include <qlabel.h>
#include <qglobal.h>

extern "C" {
	void setup_display();
}

LoginDialog::LoginDialog(bool standAlone,bool disableConfiguration, const QString& config): LoginDialog_Base() { 
	m_disableConfiguration=disableConfiguration || (!config.isEmpty());
	m_exitOnLoss=false;
	m_standAlone=standAlone;	
	m_connection=new KNXConnection();
	m_initError=false;
//	m_widget=new LoginDialog_Base(this);
//	setMainWidget(m_widget);
	connect(okbutton,SIGNAL(clicked()),this,SLOT(slotOk()));
	connect(cancelbutton,SIGNAL(clicked()),this,SLOT(slotCancel()));
	connect(used_connection,SIGNAL(activated(const QString&)),this,SLOT(connectionChanged(const QString&)));
	if (m_disableConfiguration) {
		settingsbutton->setEnabled(false);
		settingsbutton->hide();
	}
	connect(settingsbutton,SIGNAL(clicked()),this,SLOT(slotSettings()));

	fillInConnectionCombo();
	if (!config.isEmpty()) {
		bool configFound=false;
		used_connection->hide();
		connectionlabel->hide();
		for (unsigned int i=0;i<used_connection->count();i++) {
			if (used_connection->text(i)==config) {
				configFound=true;
				used_connection->setCurrentItem(i);
				connectionChanged(config);
				setCaption(i18n("%1 - kNX Login").arg(config));
				break;
			}	
		}
		if (!configFound) m_initError=true;
	}
	setFixedSize(sizeHint());
}

LoginDialog::~LoginDialog() {
}

void LoginDialog::setStandAlone() {
	m_standAlone=true;
}

void LoginDialog::fillInConnectionCombo() {
	used_connection->clear();
	if (!m_disableConfiguration) used_connection->insertItem(i18n("<New Connection>"));
	QDir dir(QDir::homeDirPath()+(m_standAlone?"/.nx_dm/config":"/.nx/config"),"*.nxs");
	for (unsigned int i=0;i<dir.count();i++) {
		QString conn=dir[i];
		if (conn.compare("nxclient.conf")==0) continue;
		used_connection->insertItem(conn.left(conn.length()-4));
	}
	if (used_connection->count()<(m_disableConfiguration?1:2)) okbutton->setEnabled(false);
	connectionChanged(used_connection->currentText());
}

void LoginDialog::connectionChanged(const QString& name) {
	if (!m_disableConfiguration) {
		okbutton->setEnabled(used_connection->currentItem()>0);
	}
	m_connection->load(QDir::homeDirPath()+(m_standAlone?"/.nx_dm/config/":"/.nx/config/")+name+".nxs");
}

void LoginDialog::slotOk() {
	if (used_connection->currentItem()>-1) {
		m_connection->setNXLogin(login_password->text(),login_name->text());
		if (m_standAlone) {
			hide();
			connect(m_connection,SIGNAL(proxyStarted()),this,SLOT(enableExitOnConnectionLoss()));
			connect(m_connection,SIGNAL(destroyed()),this,SLOT(connectionLost()));
		}
		if (!m_connection->doConnect(used_connection->currentText())) {
			show();
			disconnect(m_connection,SIGNAL(proxyStarted()),this,SLOT(enableExitOnConnectionLoss()));
			disconnect(m_connection,SIGNAL(destroyed()),this,SLOT(connectionLost()));
		} else {
			if (!m_standAlone) {
				m_connection=new KNXConnection();
				connectionChanged(used_connection->currentText());
			}
		}

		//this->hide();
	}
}

void LoginDialog::enableExitOnConnectionLoss() {
	m_exitOnLoss=true;
}

void LoginDialog::connectionLost() {
	if (m_exitOnLoss) close();
	else show();
}

void LoginDialog::slotCancel() {
	close();
}

void LoginDialog::slotSettings() {
	Q_ASSERT(m_disableConfiguration==false);
	KNXConnectionSettingsDialog *dialog=new KNXConnectionSettingsDialog(QDir::homeDirPath()+(m_standAlone?"/.nx_dm/config/":"/.nx/config/"),
		(used_connection->currentItem()==0)?(QString::null):(used_connection->currentText()));
	connect(dialog,SIGNAL(filesModified()),this,SLOT(fillInConnectionCombo()));
	dialog->show();
}

void LoginDialog::closeEvent ( QCloseEvent * e ) {
	LoginDialog_Base::closeEvent(e);
	kapp->quit();
}

#include "logindialog.moc"
