/***************************************************************************
 *   Copyright (C) 2004 by Joseph Wenninger                                *
 *   jowenn@kde.org                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>

#include "knx.h"
#include <dcopclient.h>
#include <kaboutdata.h>
#include <kcmdlineargs.h>
#include <klocale.h>
#include "logindialog.h"
#include <kdebug.h>
extern "C" {
#include "include/BadWM.h"
}

#define DEBUG

static const char description[] =
    I18N_NOOP("A KDE Application");

static const char version[] = "0.1";

static KCmdLineOptions options[] =
{
//    { "+[URL]", I18N_NOOP( "Document to open." ), 0 },
    { "standalone", I18N_NOOP( "Use embedded window manager, THIS HAS TO BE THE FIRST PARAMETER" ), 0 },
    { "disable-configure", I18N_NOOP( "Disable the configuration button" ), 0 },
    { "connection <connection>", I18N_NOOP( "Use specified connection. The file has to be in $HOME/.nx/config and needs to be called <connection>.conf" ), 0 },
    KCmdLineLastOption
};


extern "C" {
int wm_main(int argc, char **argv);
}

int main(int argc, char **argv)
{
    bool standAlone=false;
    if (argc>1) {
	    if (strcmp(argv[1],"--standalone")==0) {
		    pid_t child=fork();
		    if (child<0) exit(1);
		    if (child==0) {
			exit(wm_main(0,0));
    			}
		    standAlone=true;
		}
	}

    KAboutData about("knx", I18N_NOOP("KNX"), version, description,
                     KAboutData::License_GPL, "(C) 2004 Joseph Wenninger", 0, 0, "jowenn@kde.org");
    about.addAuthor( "Joseph Wenninger", 0, "jowenn@kde.org" );
    KCmdLineArgs::init(argc, argv, &about);
    KCmdLineArgs::addCmdLineOptions(options);
    KApplication::disableAutoDcopRegistration();
    KApplication app;
    KCmdLineArgs* args = KCmdLineArgs::parsedArgs();


    QString config="";
    if(args->isSet("connection")) {
	kdDebug()<<"config is set"<<endl;
	config=QString::fromLocal8Bit(args->getOption("connection"));
    }
    kdDebug()<<"--connection parameter="<<config<<endl;
    LoginDialog *ld=new LoginDialog(standAlone,args->isSet("disable-configure"),config);
    if (ld->initError()) exit(1);
    ld->show();
    //setup_display();
    //app.allSetup=true;
    return app.exec();
}

/* BadWM - minimalistic window manager for the X Window System.
 * Copyright (C) 2003-2004 Robert Annessi <robert@annessi.at>
 * see README for license and other details. */

#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <X11/cursorfont.h>
#include <stdio.h>
#include "include/config.h"

Display		*dpy;
int 		num_screens;
ScreenInfo	*screens;
ScreenInfo	*current_screen;
Client		*current = NULL;
Window		initialising = None;
XFontStruct	*font;
Client		*head_client;
Atom		xa_wm_state;
Atom		xa_wm_change_state;
Atom		xa_wm_protos;
Atom		xa_wm_delete;
Atom		xa_wm_cmapwins;
Cursor          arrow_curs;
Cursor          move_curs;
Cursor          resize_cursors[8];
const char	*opt_display = "";
const char	*opt_font = DEF_FONT;
const char	*opt_fg = DEF_FG;
const char	*opt_bg = DEF_BG;
const char	**opt_term = NULL;
const char	*opt_fc = DEF_FC;
int		opt_bw = DEF_BW;
int		vdesk = KEY_TO_VDESK(XK_1);
int		opt_snap = DEF_SNAP;
int		quitting = 0;
unsigned int numlockmask = 0;
static void *xmalloc(size_t size);

static void *xmalloc(size_t size) {
	       void *ptr = malloc(size);
	       if (!ptr) {
               exit(1);
       }
       return ptr;
}

extern "C" {
int wm_main(int argc, char **argv) {
	struct sigaction act;
	XEvent ev;
	int i; 

        for (i = 1; i < argc; i++) {
                if (!strcmp(argv[i], "-display") && i+1<argc)
                        opt_display = argv[++i];
                else if (!strcmp(argv[i], "-fg") && i+1<argc)
                        opt_fg = argv[++i];
                else if (!strcmp(argv[i], "-bg") && i+1<argc)
                        opt_bg = argv[++i];
                else if (!strcmp(argv[i], "-fc") && i+1<argc)
                        opt_fc = argv[++i];
                else if (!strcmp(argv[i], "-term") && i+1<argc) {
                        opt_term = (const char **)xmalloc(3 * sizeof(const char *));
                        opt_term[0] = argv[++i];
                        opt_term[1] = opt_term[0];
                        opt_term[2] = NULL;
                } else if (!strcmp(argv[i], "-version")) {
                        printf(PACKAGE_NAME"-"VERSION" by <"PACKAGE_BUGREPORT">\n");
                        exit(0);
                } else {
                        printf("Usage: "PACKAGE_NAME" [-display display] [-term termprog] [-fg foreground]\n");
                        printf("\t[-fc fixedcolor] [-bg background] [-version]\n");
                        exit(2);
                }
        }

        if (!opt_term) {
                opt_term = (const char **)xmalloc(3 * sizeof(const char *));
                opt_term[0] = DEF_TERM;
                opt_term[1] = opt_term[0];
                opt_term[2] = NULL;
        }

	act.sa_handler = handle_signal;
	sigemptyset(&act.sa_mask);
	act.sa_flags = 0;
	sigaction(SIGTERM, &act, NULL);
	sigaction(SIGINT, &act, NULL);
	sigaction(SIGHUP, &act, NULL);

	printf("Setting up display\n");
	setup_display();

	/* main event loop here */
	for (;;) {
		printf("wm_main: Mainloop\n");
		XNextEvent(dpy, &ev);
		switch (ev.type) {
			case KeyPress:
				handle_key_event(&ev.xkey); break;
			case ButtonPress:
				handle_button_event(&ev.xbutton); break;
			case ConfigureRequest:
				handle_configure_request(&ev.xconfigurerequest); break;
			case MapRequest:
				handle_map_request(&ev.xmaprequest); break;
			case ClientMessage:
				handle_client_message(&ev.xclient); break;
			case EnterNotify:
				handle_enter_event(&ev.xcrossing); break;
			case PropertyNotify:
				handle_property_change(&ev.xproperty); break;
			case UnmapNotify:
				handle_unmap_event(&ev.xunmap); break;
			default:
				break;
		}
	}
	return 1;
}


void setup_display() {
	XGCValues gv;
	XSetWindowAttributes attr;
	XColor dummy;
	XModifierKeymap *modmap;
	KeySym *keysym;
	KeySym keys_to_grab[] = {
		KEY_NEXT, KEY_NEW, KEY_KILL, KEY_LOWER, KEY_MAX, KEY_FIX, KEY_MAXVERT,
		KEY_TOPLEFT, KEY_TOPRIGHT, KEY_BOTTOMLEFT, KEY_BOTTOMRIGHT,
		XK_1, XK_2, XK_3, XK_4, XK_5, XK_6, XK_7, XK_8, XK_9, KEY_MAXHORIZ,
		0
	};
	/* used in scanning windows (XQueryTree) */
	unsigned int i, j, nwins;
	Window dw1, dw2, *wins;
	XWindowAttributes winattr;

	dpy = XOpenDisplay(opt_display);
	if (!dpy) { 
		fprintf(stderr, "can't open display %s\n", opt_display);
		exit(1);
	}
	XSetErrorHandler(handle_xerror);

	xa_wm_state = XInternAtom(dpy, "WM_STATE", False);
	xa_wm_change_state = XInternAtom(dpy, "WM_CHANGE_STATE", False);
	xa_wm_protos = XInternAtom(dpy, "WM_PROTOCOLS", False);
	xa_wm_delete = XInternAtom(dpy, "WM_DELETE_WINDOW", False);

	font = XLoadQueryFont(dpy, opt_font);
	if (!font) font = XLoadQueryFont(dpy, DEF_FONT);

	arrow_curs = XCreateFontCursor(dpy, XC_top_left_arrow);
	move_curs = XCreateFontCursor(dpy, XC_fleur);

        resize_cursors[NorthWest] = XCreateFontCursor(dpy,XC_top_left_corner);
        resize_cursors[North] = XCreateFontCursor(dpy,XC_top_side);
        resize_cursors[NorthEast] = XCreateFontCursor(dpy,XC_top_right_corner);
        resize_cursors[East] = XCreateFontCursor(dpy,XC_right_side);
        resize_cursors[SouthEast] = XCreateFontCursor(dpy,XC_bottom_right_corner);
        resize_cursors[South] = XCreateFontCursor(dpy,XC_bottom_side);
        resize_cursors[SouthWest] = XCreateFontCursor(dpy,XC_bottom_left_corner);
        resize_cursors[West] = XCreateFontCursor(dpy,XC_left_side);

	/* find out which modifier is NumLock - we'll use this when grabbing
	 * every combination of modifiers we can think of */
	modmap = XGetModifierMapping(dpy);
	for (i = 0; i < 8; i++) {
		for (j = 0; j < modmap->max_keypermod; j++) {
			if (modmap->modifiermap[i*modmap->max_keypermod+j] == XKeysymToKeycode(dpy, XK_Num_Lock)) {
				numlockmask = (1<<i);
#ifdef DEBUG
                                fprintf(stderr, "setup_display() : XK_Num_Lock is (1<<0x%02x)\n", i); 
#endif  
			}
		}
	}
	XFreeModifiermap(modmap);

	/* set up GC parameters - same for each screen */
	gv.function = GXinvert;
	gv.subwindow_mode = IncludeInferiors;
	gv.line_width = opt_bw;
	gv.font = font->fid;

	/* set up root window attributes - same for each screen */
	attr.event_mask = ChildMask | PropertyChangeMask | EnterWindowMask | LeaveWindowMask
		| ButtonMask
		;

	/* now set up each screen in turn */
	num_screens = ScreenCount(dpy);
	screens = (ScreenInfo *)xmalloc(num_screens * sizeof(ScreenInfo));
	for (i = 0; i < num_screens; i++) {
		char *ds, *colon, *dot;
		ds = DisplayString(dpy);
		/* set up DISPLAY environment variable to use */
		colon = rindex(ds, ':');
		if (colon && num_screens > 1) {
			screens[i].display = (char *)xmalloc(14 + strlen(ds));
			strcpy(screens[i].display, "DISPLAY=");
			strcat(screens[i].display, ds);
			colon = rindex(screens[i].display, ':');
			dot = index(colon, '.');
			if (!dot)
				dot = colon + strlen(colon);
			snprintf(dot, 5, ".%d", i);
		} else
			screens[i].display = NULL;

		screens[i].screen = i;
		screens[i].root = RootWindow(dpy, i);


		XAllocNamedColor(dpy, DefaultColormap(dpy, i), opt_fg, &screens[i].fg, &dummy);
		XAllocNamedColor(dpy, DefaultColormap(dpy, i), opt_bg, &screens[i].bg, &dummy);
		XAllocNamedColor(dpy, DefaultColormap(dpy, i), opt_fc, &screens[i].fc, &dummy);

		screens[i].invert_gc = XCreateGC(dpy, screens[i].root, GCFunction | GCSubwindowMode | GCLineWidth | GCFont, &gv);

		XChangeWindowAttributes(dpy, screens[i].root, CWEventMask, &attr);
		for (keysym = keys_to_grab; *keysym; keysym++) {
			grab_keysym(screens[i].root, Mod4Mask, *keysym);
		}

		/* scan all the windows on this screen */
#ifdef DEBUG
                fprintf(stderr, "main:XQueryTree(); ");
#endif
		XQueryTree(dpy, screens[i].root, &dw1, &dw2, &wins, &nwins);
#ifdef DEBUG
                fprintf(stderr, "%d windows\n", nwins);
#endif

		for (j = 0; j < nwins; j++) {
			XGetWindowAttributes(dpy, wins[j], &winattr);
			if (!winattr.override_redirect && winattr.map_state == IsViewable)
				make_new_client(wins[j], &screens[i]);
		}
		XFree(wins);
		XDefineCursor(dpy,screens[i].root,arrow_curs);
	}
	current_screen = find_screen(DefaultScreen(dpy));
}


}
